@@ -17,6 +17,7 @@ without a database) are collected.
 import io
 import numpy as np
 import warnings
+from collections import namedtuple
 ####################################################################################
 
 #helper functions used in calculation, parser etc.
@@ -277,8 +278,7 @@ def interpolate_dos(
     :note: output units are in Ry!
     """
 
-    f = open_general(dosfile)
-    with f:
+    with open_general(dosfile) as f:
         text = f.readline()  # dummy readin of header, may be replaced later
         npot = int(f.readline().split()[0])
         iemax = int(f.readline().split()[0])
@@ -353,8 +353,7 @@ def get_ef_from_potfile(potfile):
     """
     extract fermi energy from potfile
     """
-    f = open_general(potfile)
-    with f:
+    with open_general(potfile) as f:
         txt = f.readlines()
     ef = float(txt[3].split()[1])
     return ef
@@ -411,14 +410,14 @@ def convert_to_fortran(val, quote_strings=True):
         else:
             val_str = '.false.'
     elif isinstance(val, numbers.Integral):
-        val_str = '{:d}'.format(val)
+        val_str = f'{val:d}'
     elif isinstance(val, numbers.Real):
-        val_str = ('{:18.10e}'.format(val)).replace('e', 'd')
+        val_str = f'{val:18.10e}'.replace('e', 'd')
     elif isinstance(val, str):
         if quote_strings:
-            val_str = "'{!s}'".format(val)
+            val_str = f"'{val!s}'"
         else:
-            val_str = '{!s}'.format(val)
+            val_str = f'{val!s}'
     else:
         raise ValueError(f"Invalid value '{val}' of type '{type(val)}' passed, accepts only booleans, ints, "
                          'floats and strings')
@@ -430,9 +429,13 @@ def convert_to_fortran_string(string):
     """
     converts some parameter strings to the format for the inpgen
     :param string: some string
-    :returns: string in right format (extra "")
+    :returns: string in right format (extra "" if not already present)
     """
-    return f'"{string}"'
+    if not string.strip().startswith("\"") or \
+       not string.strip().endswith("\""):
+        return f'"{string}"'
+    else:
+        return string
 
 
 def is_sequence(arg):
@@ -541,6 +544,56 @@ def rel_to_abs_f(vector, cell):
         return False
 
 
+def find_symmetry_relation(from_pos, to_pos, rotations, shifts, cell, relative_pos=False, film=False):
+    """
+    Find symmetry relation between the given vectors. This functions assumes
+    that a symmetry relation exists otherwise an error is raised
+
+    :param from_pos: vector to rotate
+    :param to_pos: vector to rotate to
+    :param rotations: list of np.arrays with the given symmetry rotations
+    :param shifts: list of np.arrays with the given shifts for the symmetry operations
+    :param cell: Bravais matrix of a crystal 3x3 Array, List of list or np.array
+    :param relative_pos: bool if True the given vectors are assuemd to be in internal coordinates
+    :param film: bool if True the vectors are assumed to be film coordinates
+
+    :returns: tuple of rotation and shift mapping ``from_pos`` to ``to_pos``
+
+    :raises ValueError: If no symmetry relation is found
+    """
+
+    def lattice_shifts():
+        for i in range(-2, 3):
+            for j in range(-2, 3):
+                for k in range(-2, 3):
+                    yield np.array([i, j, k], dtype=int)
+
+    if not relative_pos:
+        if film:
+            from_pos = abs_to_rel_f(from_pos, cell, (True, True, False))
+            to_pos = abs_to_rel_f(to_pos, cell, (True, True, False))
+        else:
+            from_pos = abs_to_rel(from_pos, cell)
+            to_pos = abs_to_rel(to_pos, cell)
+
+    cell = np.array(cell)
+    cell_square = np.matmul(cell.T, cell)
+
+    for rot, shift in zip(rotations, shifts):
+        rot_pos = np.matmul(rot, np.array(from_pos)) + shift
+        diff = rot_pos - np.array(to_pos)
+        for lat_shift in lattice_shifts():
+            length = np.sqrt(np.dot(np.matmul(diff + lat_shift, cell_square), diff + lat_shift))
+            if length < 1e-4:
+                return rot, shift
+
+    raise ValueError(f'No symmetry relation found between {from_pos} and {to_pos}')
+
+
+#namedtuple used for input output of atom sites
+AtomSiteProperties = namedtuple('AtomSiteProperties', ('position', 'symbol', 'kind'))
+
+
 def get_wigner_matrix(l, phi, theta):
     """Produces the wigner rotation matrix for the density matrix
 
