@@ -17,9 +17,10 @@ of fleur in a robust way.
 Essentially a low-level version of the FleurinpModifier in aiida_fleur.
 """
 from collections import namedtuple
-from lxml import etree
-import copy
+
 from masci_tools.util.xml.collect_xml_setters import XPATH_SETTERS, SCHEMA_DICT_SETTERS, NMMPMAT_SETTERS
+from masci_tools.io.io_fleurxml import load_inpxml
+from pathlib import Path
 #Enable warnings for missing docstrings
 #pylint: enable=missing-function-docstring
 
@@ -83,7 +84,6 @@ class FleurXMLModifier:
         self._tasks = []
         self.validate_signatures = validate_signatures
 
-
     @classmethod
     def fromList(cls, task_list, *args, **kwargs):
         """
@@ -98,8 +98,18 @@ class FleurXMLModifier:
         """
 
         fm = cls(*args, **kwargs)
+        fm.add_task_list(task_list)
+        return fm
 
-        facade_methods = fm.get_avail_actions()
+    def add_task_list(self, task_list):
+        """
+        Add a list of tasks to be added
+
+        :param task_list: list of tuples first index is the name of the method
+                          second is defining the arguments by keyword in a dict
+        """
+
+        facade_methods = self.get_avail_actions()
 
         for name, kwargs in task_list:
             try:
@@ -107,8 +117,6 @@ class FleurXMLModifier:
             except KeyError as exc:
                 raise ValueError(f"Unknown modification method '{name}'") from exc
 
-        return fm
-
     def _validate_signature(self, name, *args, **kwargs):
         """
         Validate that the given arguments to the registration
@@ -139,7 +147,7 @@ class FleurXMLModifier:
             except TypeError as exc:
                 raise TypeError(
                     f"The given arguments for the registration method '{name}' are not valid for the XML modifying function"
-                ) from exc
+                    f'The following error was raised: {exc}') from exc
 
     @classmethod
     def apply_modifications(cls, xmltree, nmmp_lines, modification_tasks, validate_changes=True):
@@ -180,10 +188,6 @@ class FleurXMLModifier:
                 action = cls.nmmpmat_functions[task.name]
                 nmmp_lines = action(xmltree, nmmp_lines, schema_dict, *task.args, **task.kwargs)
 
-            elif task.name in cls.no_xmltree_functions:
-                action = cls.nmmpmat_functions[task.name]
-                nmmp_lines = action(xmltree, nmmp_lines, schema_dict, *task.args, **task.kwargs)
-
             else:
                 raise ValueError(f'Unknown task {task.name}')
 
@@ -273,17 +277,10 @@ class FleurXMLModifier:
 
         :returns: a modified xmltree and if existent a modified density matrix file
         """
-        if isinstance(original_inpxmlfile, etree._ElementTree):
-            original_xmltree = original_inpxmlfile
-        else:
-            parser = etree.XMLParser(attribute_defaults=True, encoding='utf-8')
-            try:
-                original_xmltree = etree.parse(original_inpxmlfile, parser)
-            except etree.XMLSyntaxError as msg:
-                raise ValueError(f'Failed to parse input file: {msg}') from msg
+        original_xmltree, _ = load_inpxml(original_inpxmlfile)
 
         if original_nmmp_file is not None:
-            if isinstance(original_nmmp_file, str):
+            if isinstance(original_nmmp_file, (str, Path)):
                 with open(original_nmmp_file, mode='r') as n_mmp_file:
                     original_nmmp_lines = n_mmp_file.read().split('\n')
             else:
