@@ -12,11 +12,18 @@
 ###############################################################################
 """Convenience tools only related to Python Standard Library."""
 
-import copy
-import typing
-from dataclasses import field
-import json
-import re
+import copy as _copy
+import dataclasses as _dc
+import datetime as _datetime
+import json as _json
+import random as _random
+import re as _re
+import string as _string
+import sys as _sys
+import typing as _typing
+
+import humanfriendly as _humanfriendly
+import pytz as _pytz
 
 
 def is_number(a_string: str) -> bool:
@@ -34,7 +41,7 @@ def is_number(a_string: str) -> bool:
         return False
 
 
-def to_number(a_string: str) -> object:
+def to_number(a_string: str) -> _typing.Union[int, float, str]:
     """Converts a string representation of a number into a numerical type.
 
     Numbers with decimals below machine epsilon, usually around 1e-16, get recognized as int.
@@ -52,11 +59,9 @@ def to_number(a_string: str) -> object:
     return num
 
 
-def now():
+def now() -> _datetime.datetime:
     """Get now time localized to UTC."""
-    from datetime import datetime
-    import pytz
-    return datetime.now(tz=pytz.UTC)
+    return _datetime.datetime.now(tz=_pytz.UTC)
     # same as: pytz.UTC.localize(datetime.now())
     # not same as: datetime.now(tz=timezone.utc)
 
@@ -75,41 +80,36 @@ def random_string(length: int,
     :param whitespace: True: include whitespaces `\t\n\r\x0b\x0c`.
     :return: random string of length.
     """
-    import string
-    import random
-
     character_sets = {
-        string.ascii_uppercase: ascii_uppercase,
-        string.ascii_lowercase: ascii_lowercase,
-        string.digits: digits,
-        string.whitespace: whitespace
+        _string.ascii_uppercase: ascii_uppercase,
+        _string.ascii_lowercase: ascii_lowercase,
+        _string.digits: digits,
+        _string.whitespace: whitespace
     }
-    population = ''.join([characters for characters, included in character_sets.items() if included])
-    return ''.join(random.choices(population=population, k=length))
+    population = ''.join(characters for characters, included in character_sets.items() if included)
+
+    return ''.join(_random.choices(population=population, k=length))
 
 
-def enforce_minimum_python_version(minimum_version: tuple = (3, 7)):
+def enforce_minimum_python_version(minimum_version: _typing.Tuple[int, int] = (3, 7)):
     """For some operation requiring a minimum version, enforce for interpreter.
 
     :param minimum_version:
     :return: 0 if satisfied, sys.exit if not
     """
-    import sys
-    current_version = sys.version_info
-    if current_version[0] == minimum_version[0] and current_version[1] >= minimum_version[1]:
-        pass
-    else:
-        sys.stderr.write('[%s] - Error: Your Python interpreter must be %d.%d or greater (within major version %d)\n' %
-                         (sys.argv[0], minimum_version[0], minimum_version[1], minimum_version[0]))
-        sys.exit(-1)
+    current_version = _sys.version_info
+    if (current_version[0] != minimum_version[0] or current_version[1] < minimum_version[1]):
+        _sys.stderr.write('[%s] - Error: Your Python interpreter must be %d.%d or greater (within major version %d)\n' %
+                          (_sys.argv[0], minimum_version[0], minimum_version[1], minimum_version[0]))
+        _sys.exit(-1)
     return 0
 
 
-def flatten_list_of_lists(list_of_lists: list):
+def flatten_list_of_lists(list_of_lists: _typing.List[_typing.List[_typing.Any]]) -> _typing.List[_typing.Any]:
     return [item for sublist in list_of_lists for item in sublist]
 
 
-def split_list(lst: list, n: int):
+def split_list(lst: list, n: int) -> _typing.Generator[_typing.Any, None, None]:
     """Yield n-sized chunks/batches from lst. Last chunk has remainder size.
 
     :param lst: list
@@ -126,7 +126,9 @@ def split_list(lst: list, n: int):
         yield lst[i:i + n]
 
 
-def sort_lists_by_list(lists: typing.List[typing.List], key_list_index: int = 0, desc: bool = False):
+def sort_lists_by_list(lists: _typing.List[_typing.List],
+                       key_list_index: int = 0,
+                       desc: bool = False) -> _typing.Iterator[_typing.Tuple]:
     """Sort multiple lists by one list.
 
     The key_list gets sorted. The other lists get sorted in the same order.
@@ -153,11 +155,11 @@ def sort_lists_by_list(lists: typing.List[typing.List], key_list_index: int = 0,
 class NoIndent(object):
     """ Value wrapper. """
 
-    def __init__(self, value):
+    def __init__(self, value: _typing.Any):
         self.value = value
 
 
-class JSONEncoderTailoredIndent(json.JSONEncoder):
+class JSONEncoderTailoredIndent(_json.JSONEncoder):
     """JSONEncoder which allows to not indent items wrapped in 'NoIndent()'.
 
     Reference: https://stackoverflow.com/a/13252112/8116031
@@ -184,24 +186,24 @@ class JSONEncoderTailoredIndent(json.JSONEncoder):
     """
 
     FORMAT_SPEC = '@@{}@@'
-    regex = re.compile(FORMAT_SPEC.format(r'(\d+)'))
+    regex = _re.compile(FORMAT_SPEC.format(r'(\d+)'))
 
     def __init__(self, **kwargs):
         # Save copy of any keyword argument values needed for use here.
         self.__sort_keys = kwargs.get('sort_keys', None)
         super().__init__(**kwargs)
 
-    def default(self, obj):  #pylint: disable=arguments-differ
-        if isinstance(obj, NoIndent):
-            return self.FORMAT_SPEC.format(id(obj))
+    def default(self, o: _typing.Any) -> _typing.Any:  # pylint: disable=arguments-differ
+        if isinstance(o, NoIndent):
+            return self.FORMAT_SPEC.format(id(o))
         else:
-            return super().default(obj)
+            return super().default(o)
 
-    def encode(self, obj):  #pylint: disable=arguments-differ
+    def encode(self, o: _typing.Any) -> str:  # pylint: disable=arguments-differ
         from _ctypes import PyObj_FromPtr
 
         format_spec = self.FORMAT_SPEC  # Local var to expedite access.
-        json_repr = super().encode(obj)  # Default JSON.
+        json_repr = super().encode(o)  # Default JSON.
 
         # Replace any marked-up object ids in the JSON repr with the
         # value returned from the json.dumps() of the corresponding
@@ -210,7 +212,7 @@ class JSONEncoderTailoredIndent(json.JSONEncoder):
             # see https://stackoverflow.com/a/15012814/355230
             id_obj = int(match.group(1))
             no_indent = PyObj_FromPtr(id_obj)
-            json_obj_repr = json.dumps(no_indent.value, sort_keys=self.__sort_keys)
+            json_obj_repr = _json.dumps(no_indent.value, sort_keys=self.__sort_keys)
 
             # Replace the matched id string with json formatted representation
             # of the corresponding Python object.
@@ -219,7 +221,7 @@ class JSONEncoderTailoredIndent(json.JSONEncoder):
         return json_repr
 
 
-def dataclass_default_field(obj: typing.Any, deepcopy=True) -> typing.Any:
+def dataclass_default_field(obj: _typing.Any, deepcopy=True) -> _typing.Any:
     """Abbreviator for defining mutable types as default values for fields of dataclasses.
     References:
     - https://stackoverflow.com/a/53870411/8116031
@@ -244,9 +246,9 @@ def dataclass_default_field(obj: typing.Any, deepcopy=True) -> typing.Any:
     >>>     dataclass_default_field({"a": (1, "x"), "b": (1, "y")})
     """
     if deepcopy:
-        return field(default_factory=lambda: copy.deepcopy(obj))
+        return _dc.field(default_factory=lambda: _copy.deepcopy(obj))
     else:
-        return field(default_factory=lambda: copy.copy(obj))
+        return _dc.field(default_factory=lambda: _copy.copy(obj))
 
 
 class SizeEstimator:
@@ -258,10 +260,8 @@ class SizeEstimator:
     """
 
     def __init__(self):
-        import sys
-        import humanfriendly
-        self.sys = sys
-        self.hf = humanfriendly
+        self.sys = _sys
+        self.hf = _humanfriendly
 
         # Custom objects know their class.
         # Function objects seem to know way too much, including modules.
@@ -269,7 +269,7 @@ class SizeEstimator:
         from types import ModuleType, FunctionType
         self.BLACKLIST = type, ModuleType, FunctionType
 
-    def sizeof_simple(self, obj) -> tuple:
+    def sizeof_simple(self, obj: _typing.Any) -> _typing.Tuple[int, str]:
         """For simple PSL data structures. Reference: https://stackoverflow.com/a/19865746/8116031
 
         :return: tuple of bytesize, humanfriendly bytesize
@@ -277,7 +277,7 @@ class SizeEstimator:
         size = self.sys.getsizeof(obj)
         return size, self.hf.format_size(size)
 
-    def sizeof_via_blacklist(self, obj):
+    def sizeof_via_blacklist(self, obj: _typing.Any) -> _typing.Tuple[int, str]:
         """sum size of object & members. Reference: https://stackoverflow.com/a/30316760/8116031."""
         from gc import get_referents
 
@@ -288,7 +288,7 @@ class SizeEstimator:
         objects = [obj]
         while objects:
             need_referents = []
-            for obj in objects:  #pylint: disable=redefined-argument-from-local
+            for obj in objects:  # pylint: disable=redefined-argument-from-local
                 if not isinstance(obj, self.BLACKLIST) and id(obj) not in seen_ids:
                     seen_ids.add(id(obj))
                     size += self.sys.getsizeof(obj)
@@ -297,7 +297,7 @@ class SizeEstimator:
 
         return size, self.hf.format_size(size)
 
-    def sizeof_via_whitelist(self, obj):
+    def sizeof_via_whitelist(self, obj: _typing.Any) -> _typing.Tuple[int, str]:
         """Recursively iterate to sum size of object & members.
 
         'gives much more fine-grained control over the types we're going to count for memory usage,
@@ -306,7 +306,8 @@ class SizeEstimator:
         Reference: https://stackoverflow.com/a/30316760/8116031.
         """
         from numbers import Number
-        from collections import Set, Mapping, deque  #pylint: disable=no-name-in-module
+        from collections import deque  # pylint: disable=no-name-in-module
+        from collections.abc import Set, Mapping
 
         zero_depth_bases = (str, bytes, Number, range, bytearray)
         _seen_ids = set()
