@@ -266,8 +266,9 @@ class BokehPlotter(Plotter):
 
         #Rename replaced keys back to standard names
         for key, replace_key in kwargs.items():
-            custom_val = plot_kwargs.pop(replace_key)
-            plot_kwargs[key] = custom_val
+            custom_val = plot_kwargs.pop(replace_key, None)
+            if custom_val is not None:
+                plot_kwargs[key] = custom_val
 
         if not post_process:
             return plot_kwargs
