@@ -296,7 +296,7 @@ def plot_fleur_dos(dosdata,
                    attributes,
                    spinpol=True,
                    bokeh_plot=False,
-                   multiply_by_equiv_atoms=False,
+                   multiply_by_equiv_atoms=True,
                    plot_keys=None,
                    show_total=True,
                    show_interstitial=True,
